#!/usr/bin/env python3
import argparse
import logging
import sys

from opengrokfs.opengrok import OpenGrok


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--debug', action='store_true')
    parser.add_argument('--url', required=True)
    parser.add_argument('--project', action='append')
    parser.add_argument('--strip')

    parser.add_argument('-n', '--line-number', action='store_true')
    parser.add_argument('pattern')

    try:
        with open('.opengrokfs') as f:
            argv = [l.rstrip() for l in f.readlines()]
    except FileNotFoundError:
        argv = []

    args = parser.parse_args(argv + sys.argv[1:])

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    og = OpenGrok(args.url, projects=args.project, strip=args.strip)
    results = og.search({'q': args.pattern})

    for r in results:
        if args.line_number:
            print('%s:%d:%s' % (r.path, r.line, r.text))
        else:
            print('%s:%s' % (r.path, r.text))

    sys.exit(0 if results else 1)


if __name__ == '__main__':
    main()



import urllib.parse

import requests
import dateparser
import attr
from lxml import etree


@attr.s
class File(object):
    name = attr.ib()
    date = attr.ib()
    size = attr.ib()
    dir = attr.ib(False)


@attr.s
class Result(object):
    path = attr.ib()
    function = attr.ib(None)
    line = attr.ib(0)
    text = attr.ib(None)


class OpenGrok(object):
    def __init__(self, url, projects=None, strip=None):
        if not url.endswith('/'):
            url += '/'
        self.url = url
        self.base = urllib.parse.urlparse(self.url).path
        self.htmlparser = etree.HTMLParser()

        self.params = {'n': 10000}
        self.projects = projects
        if projects:
            self.params['project'] = projects

        self.strip = strip

    def single_project(self):
        return self.projects and len(self.projects) == 1 and self.projects[0]

    def _text(self, item):
        return ''.join(item.itertext())

    def search(self, query):
        params = dict(self.params)
        params.update(query)

        page = requests.get(self.url + '/search', params)

        tree = etree.fromstring(page.text.encode('utf-8'), self.htmlparser)

        if '<title>Search Error</title>' in page.text:
            msgp = tree.find('.//div[@id="results"]/p')
            msg = msgp.text if msgp is not None else 'Search Error'
            raise Exception(msg)

        results = []
        for tr in tree.findall('.//div[@id="results"]//tr'):
            if tr.get('class') == 'dir':
                continue

            href = tr.find('./td[@class="f"]/a').get('href')
            path = href.replace(self.base, '').replace('xref', '')

            if self.strip and path.startswith(self.strip):
                path = path[len(self.strip):]

            anchors = tr.findall('./td/tt/a[@class="s"]')
            for a in anchors:
                line = int(a[0].text)
                text = ''.join(a.itertext())
                text = text.replace('%d ' % line, '')

                results.append(Result(path=path, line=line, text=text))

            if not anchors:
                results.append(Result(path=path))

        return results

    def get(self, path):
        page = requests.get(self.url + '/raw/' + path)
        return page.text

    def parse_size(self, sizetext):
        size = float(sizetext.split(' ')[0])
        if 'KiB' in sizetext:
            # Precision has been lost due to rounding, so just try to
            # increase the size and hope nothing breaks
            size += 0.1
            size *= 1024

        return int(size)

    def list(self, path='/'):
        page = requests.get(self.url + '/xref/' + path + '/')
        tree = etree.fromstring(page.text.encode('utf-8'), self.htmlparser)

        tbl = tree.find('.//table[@id="dirlist"]')
        colmap = dict([(col.text, i)
                       for i, col
                       in enumerate(tbl.findall('.//th'))
                       if col.text])

        files = []
        for tr in tbl.findall('./tbody/tr'):
            tds = tr.findall('./td')

            name = self._text(tds[colmap['Name']])
            if name.endswith('/'):
                name = name[:-1]
                directory = True
            elif name == '..':
                directory = True
            else:
                directory = False

            # Empty directories are folded in newer versions
            name = name.split('/')[0]

            date = dateparser.parse(self._text(tds[colmap['Date']]))
            date = date.replace(hour=0, minute=0, second=0, microsecond=0)

            sizetext = self._text(tds[colmap['Size']])
            # &nbsp; seems to be converted to \xa0
            sizetext = sizetext.replace(',', '.').replace('\xa0', '')
            size = self.parse_size(sizetext)

            files.append(File(name=name, date=date, size=size, dir=directory))

        return files



#!/usr/bin/env python3
import argparse
import cmd
import logging
import sys

from opengrokfs.opengrok import OpenGrok


class OpenGrokCscope(cmd.Cmd):
    prompt = '>> '

    def __init__(self, opengrok):
        super().__init__()
        self.opengrok = opengrok

    def do_q(self, arg):
        return True

    def default(self, line):
        try:
            field = int(line[0])
        except ValueError:
            print("cscope: unknown command '%s'" % line)
            return

        q = line[1:]

        if field == 0:
            params = {'refs': q}
        elif field == 1:
            params = {'defs': q}
        elif field == 4:
            params = {'q': q}
        elif field == 7:
            params = {'path': q}
        else:
            params = None

        results = []
        if params:
            results = self.opengrok.search(params)

        print('cscope: %d lines' % len(results))

        for r in results:
            function = r.function if r.function else '<unknown>'
            line = r.line if r.line else 1
            text = r.text if r.text else '<unknown>'

            print('%s %s %d %s' % (r.path, function, line, text))


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--debug', action='store_true')
    parser.add_argument('--url')
    parser.add_argument('--project', action='append')
    parser.add_argument('--strip')

    # Ignored, for compatibility with cscope
    parser.add_argument('-d')
    parser.add_argument('-f')
    parser.add_argument('-l')

    try:
        with open('.opengrokfs') as f:
            argv = [l.rstrip() for l in f.readlines()]
    except FileNotFoundError:
        argv = []

    args = parser.parse_args(argv + sys.argv[1:])

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    og = OpenGrok(args.url, projects=args.project, strip=args.strip)
    OpenGrokCscope(og).cmdloop()


if __name__ == '__main__':
    main()



from setuptools import setup


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='opengrokfs',
    description='OpenGrok FUSE filesystem and tools',
    long_description=readme(),
    license='MIT',
    url='https://github.com/rabinv/opengrokfs',
    author='Rabin Vincent',
    author_email='rabin@rab.in',
    version='0.1.0',
    install_requires=[
        'attrs>=16.3.0',
        'dateparser>=0.6.0',
        'fusepy>=2.0.4',
        'lxml>=3.7.1',
        'requests>=2.12.4',
        'requests_cache>=0.4.13',
    ],
    tests_require=['pytest'],
    entry_points={
        'console_scripts': [
            'opengrokfs = opengrokfs.opengrokfs:main',
            'oggrep = opengrokfs.grep:main',
            'ogcscope = opengrokfs.cscope:main'
        ],
    },
    packages=['opengrokfs'],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Software Development',
    ],
)



#!/usr/bin/env python3
import argparse
import datetime
from functools import lru_cache
import logging
import os.path
import errno
import stat

from fuse import FUSE, FuseOSError, Operations, LoggingMixIn
import requests_cache

from opengrokfs.opengrok import File, OpenGrok


class OpenGrokFs(LoggingMixIn, Operations):
    def __init__(self, opengrok):
        self.opengrok = opengrok
        self.pseudofile = '.opengrokfs'
        self.files = {}
        self.data = {}
        self.fd = 0
        self.list = {}
        self.today = datetime.datetime.today()
        self.files['/'] = File('', date=self.today, dir=True, size=0)

    @lru_cache(maxsize=4096)
    def _get_file(self, path):
        self._readdir(os.path.dirname(path) + '/')
        try:
            file = self.files[path]
        except KeyError:
            raise FuseOSError(errno.ENOENT)

        return file

    def getattr(self, path, fh=None):
        file = self._get_file(path)

        info = {
            'st_mode': 0o644 | (stat.S_IFDIR if file.dir else stat.S_IFREG),
            'st_ctime': file.date.timestamp(),
            'st_nlink': 2 if file.dir else 1,
            'st_size': 0 if file.dir else file.size,
        }

        info['st_mtime'] = info['st_atime'] = info['st_ctime']
        return info

    def _get_projects(self):
        return [f.name for f in self._readdir('/') if f.dir]

    @lru_cache(maxsize=4096)
    def _read(self, path):

        parts = path.strip('/').split('/')
        if self.pseudofile and parts[-1] == self.pseudofile:
            if len(parts) == 1:
                strip = '/'
                projects = self._get_projects()
            else:
                projects = [parts[0]]
                strip = '/' + '/'.join(parts[:-1]) + '/'

            args = ['--url', self.opengrok.url,
                    '--strip', strip]
            for project in projects:
                args.extend(('--project', project))

            data = '\n'.join(args).encode('utf-8')
        else:
            data = self.opengrok.get(path).encode('utf-8')

        file = self._get_file(path)
        if len(data) < file.size:
            # file.size reports larger sizes to compensate rounding in the OpenGrok UI,
            # fill the remaining bytes with whitespace instead of leaving it unfilled
            # to print junk and errors in vim and gedit due to \0 bytes
            adjust = file.size - len(data)
            data += ((adjust - 1) * ' ' + '\n').encode('utf-8')

        return data

    def read(self, path, size, offset, fh):
        data = self._read(path)
        return data[offset:offset + size]

    @lru_cache(maxsize=4096)
    def _readdir(self, path):
        files = self.opengrok.list(path)

        if self.pseudofile:
            files.append(File(self.pseudofile, date=self.today, size=1024))

        for file in files:
            self.files[path + file.name] = file

        return files

    def readdir(self, path, fh):
        if not path.endswith('/'):
            path += '/'

        return ['.'] + [f.name for f in self._readdir(path)]


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--debug', action='store_true')
    parser.add_argument('--cache', nargs='?', const='opengrokfs',
                        help="Cache network requests persistently")
    parser.add_argument('url')
    parser.add_argument('mountpoint')

    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if args.cache:
        requests_cache.install_cache(args.cache)

    FUSE(OpenGrokFs(OpenGrok(args.url)), args.mountpoint, foreground=True)


if __name__ == '__main__':
    main()
